@@ -236,36 +236,115 @@ cups_setPasswordCB (PyObject *self, PyObject *args)
   return Py_None;
 }
 
+static PyObject *
+cups_require (PyObject *self, PyObject *args)
+{
+  const char *version = VERSION;
+  const char *required;
+  const char *pver, *preq;
+  char *end;
+  unsigned long nreq, nver;
+
+  if (!PyArg_ParseTuple (args, "s", &required))
+    return NULL;
+
+  pver = version;
+  preq = required;
+  nreq = strtoul (preq, &end, 0);
+  while (preq != end)
+  {
+    preq = end;
+    if (*preq == '.')
+      preq++;
+
+    nver = strtoul (pver, &end, 0);
+    if (pver == end)
+      goto fail;
+    else {
+      pver = end;
+      if (*pver == '.')
+	pver++;
+    }
+
+    if (nver < nreq)
+      goto fail;
+
+    nreq = strtoul (preq, &end, 0);
+  }
+
+  return Py_None;
+fail:
+  PyErr_SetString (PyExc_RuntimeError, "I am version " VERSION);
+  return NULL;
+}
+
 static PyMethodDef CupsMethods[] = {
   { "modelSort", cups_modelSort, METH_VARARGS,
-    "Sort two model strings." },
+    "modelSort(s1,s2) -> integer\n\n"
+    "Sort two model strings.\n\n"
+    "@type s1: string\n"
+    "@param s1: first string\n"
+    "@type s2: string\n"
+    "@param s2: second string\n"
+    "@return: strcmp-style comparision result"},
 
   { "setUser", cups_setUser, METH_VARARGS,
-    "Set user to connect as." },
+    "setUser(user) -> None\n\n"
+    "Set user to connect as.\n\n"
+    "@type user: string\n"
+    "@param user: username"},
 
   { "setServer", cups_setServer, METH_VARARGS,
-    "Set server to connect to." },
+    "setServer(server) -> None\n\n"
+    "Set server to connect to.\n\n"
+    "@type server: string\n"
+    "@param server: server hostname" },
 
   { "setPort", cups_setPort, METH_VARARGS,
-    "Set IPP port to connect to." },
+    "setPort(port) -> None\n\n"
+    "Set IPP port to connect to.\n\n"
+    "@type port: integer\n"
+    "@param port: IPP port" },
 
   { "setEncryption", cups_setEncryption, METH_VARARGS,
-    "Set encryption policy." },
+    "setEncryption(policy) -> None\n\n"
+    "Set encryption policy.\n\n"
+    "@type policy: integer\n"
+    "@param policy: L{HTTP_ENCRYPT_ALWAYS}, L{HTTP_ENCRYPT_IF_REQUESTED}, \n"
+    "L{HTTP_ENCRYPT_NEVER}, or L{HTTP_ENCRYPT_REQUIRED}" },
 
   { "getUser", (PyCFunction) cups_getUser, METH_NOARGS,
-    "Get user to connect as." },
+    "getUser() -> string\n\n"
+    "@return: user to connect as." },
 
   { "getServer", (PyCFunction) cups_getServer, METH_NOARGS,
-    "Get server to connect to." },
+    "getServer() -> string\n\n"
+    "@return: server to connect to." },
 
   { "getPort", (PyCFunction) cups_getPort, METH_NOARGS,
-    "Get IPP port to connect to." },
+    "getPort() -> integer\n\n"
+    "@return: IPP port to connect to." },
 
   { "getEncryption", (PyCFunction) cups_getEncryption, METH_NOARGS,
-    "Get encryption policy." },
+    "getEncryption() -> integer\n\n"
+    "Get encryption policy.\n"
+    "@see: L{setEncryption}" },
 
   { "setPasswordCB", cups_setPasswordCB, METH_VARARGS,
-    "Set user to connect as." },
+    "setPasswordCB(fn) -> None\n\n"
+    "Set password callback function.  This Python function will be called \n"
+    "when a password is required.  It must take one string parameter \n"
+    "(the password prompt) and it must return a string (the password).  To \n"
+    "abort the operation it may return the empty string ('').\n\n"
+    "@type fn: callable object\n"
+    "@param fn: callback function" },
+
+  { "require", cups_require, METH_VARARGS,
+    "require(version) -> None\n\n"
+    "Require pycups version.\n\n"
+    "@type version: string\n"
+    "@param version: minimum pycups version required\n"
+    "@raise RuntimeError: requirement not met" },  
 
   { NULL, NULL, 0, NULL }
 };
@@ -275,6 +354,7 @@ initcups (void)
 {
   PyObject *m = Py_InitModule ("cups", CupsMethods);
   PyObject *d = PyModule_GetDict (m);
+  PyObject *obj;
 
   // Connection type
   cups_ConnectionType.tp_new = PyType_GenericNew;
@@ -364,7 +444,7 @@ initcups (void)
   INT_CONSTANT (CUPS_PRINTER_DELETE);
   INT_CONSTANT (CUPS_PRINTER_NOT_SHARED);
   INT_CONSTANT (CUPS_PRINTER_AUTHENTICATED);
-  //INT_CONSTANT (CUPS_PRINTER_COMMANDS);
+  INT_CONSTANT (CUPS_PRINTER_COMMANDS);
   INT_CONSTANT (CUPS_PRINTER_OPTIONS);
 
   // HTTP encryption
@@ -488,15 +568,40 @@ initcups (void)
   STR_CONSTANT (CUPS_SERVER_REMOTE_PRINTERS);
   STR_CONSTANT (CUPS_SERVER_SHARE_PRINTERS);
   STR_CONSTANT (CUPS_SERVER_USER_CANCEL_ANY);
+#ifdef HAVE_CUPS_1_3
+  STR_CONSTANT (CUPS_SERVER_REMOTE_ANY);
+#endif /* CUPS 1.3 */
 
   // Exceptions
-  HTTPError = PyErr_NewException ("cups.HTTPError", NULL, NULL);
+  obj = PyDict_New ();
+  PyDict_SetItemString (obj, "__doc__", PyString_FromString(
+    "This exception is raised when an HTTP problem has occurred.  It \n"
+    "provides an integer HTTP status code.\n\n"
+    "Use it like this::\n"
+    "  try:\n"
+    "    ...\n"
+    "  except cups.HTTPError (status):\n"
+    "    print 'HTTP status is %d' % status\n"));
+  HTTPError = PyErr_NewException ("cups.HTTPError", NULL, obj);
+  Py_DECREF (obj);
   if (HTTPError == NULL)
     return;
   Py_INCREF (HTTPError);
   PyModule_AddObject (m, "HTTPError", HTTPError);
 
-  IPPError = PyErr_NewException ("cups.IPPError", NULL, NULL);
+  obj = PyDict_New ();
+  PyDict_SetItemString (obj, "__doc__", PyString_FromString(
+    "This exception is raised when an IPP error has occurred.  It \n"
+    "provides an integer IPP status code, and a human-readable string \n"
+    "describing the error.\n\n"
+    "Use it like this::\n"
+    "  try:\n"
+    "    ...\n"
+    "  except cups.IPPError (status, description):\n"
+    "    print 'IPP status is %d' % status\n"
+    "    print 'Meaning:', description\n"));
+  IPPError = PyErr_NewException ("cups.IPPError", NULL, obj);
+  Py_DECREF (obj);
   if (IPPError == NULL)
     return;
   Py_INCREF (IPPError);
