@@ -1,6 +1,6 @@
 /*
  * cups - Python bindings for CUPS
- * Copyright (C) 2006  Tim Waugh <twaugh@redhat.com>
+ * Copyright (C) 2006, 2007  Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,17 @@ extern void debugprintf (const char *fmt, ...) FORMAT ((__printf__, 1, 2));
 #define HAVE_CUPS_1_2 1
 #endif
 
+#if (CUPS_VERSION_MAJOR > 1) || (CUPS_VERSION_MINOR > 2)
+#define HAVE_CUPS_1_3 1
+#else
+#define cupsAdminGetServerSettings _cupsAdminGetServerSettings
+#define cupsAdminSetServerSettings _cupsAdminSetServerSettings
+#endif
+
+#if (CUPS_VERSION_MAJOR > 1) || (CUPS_VERSION_MINOR > 3)
+#define HAVE_CUPS_1_4 1
+#endif
+
 #ifndef HAVE_CUPS_1_2
 #error pycups requires CUPS 1.2.x
 #endif
