@@ -20,6 +20,7 @@
 #include "cupsppd.h"
 #include "cupsmodule.h"
 
+#include <ctype.h>
 #include <iconv.h>
 #include <stdbool.h>
 
@@ -100,6 +101,33 @@ ppd_encoding_is_utf8 (PPD *ppd)
   return 0;
 }
 
+static PyObject *
+cautious_PyUnicode_DecodeUTF8 (const char *str, size_t len)
+{
+  PyObject *ret = PyUnicode_DecodeUTF8 (str, len, NULL);
+  if (ret == NULL) {
+    // It wasn't UTF-8 after all.  Just make the string safe for ASCII.
+    char *safe;
+    size_t i;
+
+    PyErr_Clear ();
+    safe = malloc (len + 1);
+    for (i = 0; i < len; i++) {
+      unsigned char ch = str[i];
+      if (!isascii (ch))
+	ch = '?';
+
+      safe[i] = ch;
+    }
+    safe[i] = '\0';
+    ret = PyUnicode_DecodeUTF8 (safe, len, NULL);
+    printf ("Bad UTF-8 string \"%s\" changed to \"%s\"\n", str, safe);
+    free (safe);
+  }
+
+  return ret;
+}
+
 static PyObject *
 make_PyUnicode_from_ppd_string (PPD *ppd, const char *ppdstr)
 {
@@ -111,7 +139,7 @@ make_PyUnicode_from_ppd_string (PPD *ppd, const char *ppdstr)
   PyObject *ret;
 
   if (ppd_encoding_is_utf8 (ppd))
-    return PyUnicode_DecodeUTF8 (ppdstr, strlen (ppdstr), NULL);
+    return cautious_PyUnicode_DecodeUTF8 (ppdstr, strlen (ppdstr));
 
   cdf = *ppd->conv_from;
 
@@ -126,7 +154,7 @@ make_PyUnicode_from_ppd_string (PPD *ppd, const char *ppdstr)
     return PyErr_SetFromErrno (PyExc_RuntimeError);
   }
 
-  ret = PyUnicode_DecodeUTF8 (outbuf, origleft - outbytesleft, NULL);
+  ret = cautious_PyUnicode_DecodeUTF8 (outbuf, origleft - outbytesleft);
   free (outbuf);
   return ret;
 }
@@ -588,42 +616,69 @@ PyMethodDef PPD_methods[] =
   {
     { "localize",
       (PyCFunction) PPD_localize, METH_NOARGS,
-      "Localize PPD." },
+      "localize() -> None\n\n"
+      "Localize PPD to the current locale." },
 
     { "markDefaults",
       (PyCFunction) PPD_markDefaults, METH_NOARGS,
-      "Mark default options.  Returns number of conflicts." },
+      "markDefaults() -> None\n\n"
+      "Set (mark) all options to their default choices." },
 
     { "markOption",
       (PyCFunction) PPD_markOption, METH_VARARGS,
-      "Mark an option." },
+      "markOption(option, choice) -> integer\n\n"
+      "Set an option to a particular choice.\n\n"
+      "@type option: string\n"
+      "@param option: option keyword\n"
+      "@type choice: string\n"
+      "@param choice: option choice\n"
+      "@return: number of conflicts" },
 
     { "conflicts",
       (PyCFunction) PPD_conflicts, METH_NOARGS,
-      "Returns number of conflicts." },
+      "conflicts() -> integer\n\n"
+      "@return: number of conflicts." },
 
     { "findOption",
       (PyCFunction) PPD_findOption, METH_VARARGS,
-      "findOption(name) -> cups.Option or None." },
+      "findOption(name)\n\n"
+      "@type name: string\n"
+      "@param name: option keyword\n"
+      "@rtype: L{Option} or None\n"
+      "@return: named option, or None if not found." },
 
     { "findAttr",
       (PyCFunction) PPD_findAttr, METH_VARARGS | METH_KEYWORDS,
-      "findAttr(name, spec=None) -> cups.Attribute or None." },
+      "findAttr(name)\n\n"
+      "@type name: string\n"
+      "@param name: attribute name\n"
+      "@type spec: string\n"
+      "@keyword spec: specifier string (optional)\n"
+      "@rtype: L{Attribute} or None\n"
+      "@return: matching attribute, or None if not found." },
 
     { "findNextAttr",
       (PyCFunction) PPD_findNextAttr, METH_VARARGS | METH_KEYWORDS,
-      "findNextAttr(name, spec=None) -> cups.Attribute or None." },
+      "findNextAttr(name)\n\n"
+      "@type name: string\n"
+      "@param name: attribute name\n"
+      "@type spec: string\n"
+      "@keyword spec: specifier string (optional)\n"
+      "@rtype: L{Attribute} or None\n"
+      "@return: next matching attribute, or None if not found." },
 
     { "nondefaultsMarked",
       (PyCFunction) PPD_nondefaultsMarked, METH_NOARGS,
-      "nondefaultsMarked() -> Boolean.\n\n"
+      "nondefaultsMarked() -> boolean\n\n"
       "Returns true if any non-default option choices are marked." },
 
     { "writeFd",
       (PyCFunction) PPD_writeFd, METH_VARARGS,
-      "writeFd (fd) -> None\n"
+      "writeFd(fd) -> None\n\n"
       "Write PPD file, with marked choices as defaults, to file\n"
-      "descriptor." },
+      "descriptor.\n\n"
+      "@type fd: integer\n"
+      "@param fd: open file descriptor" },
 
     { NULL } /* Sentinel */
   };
@@ -651,7 +706,16 @@ PyTypeObject cups_PPDType =
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "PPD file",                /* tp_doc */
+    "PPD file\n"
+    "========\n"
+    "  A PPD file.\n\n"
+    "@type constraints: L{Constraint} list\n"
+    "@ivar constraints: list of constraints\n"
+    "@type attributes: L{Attribute} list\n"
+    "@ivar attributes: list of attributes\n"
+    "@type optionGroups: L{Group} list\n"
+    "@ivar optionGroups: list of PPD option groups\n"
+    "",                        /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
@@ -854,7 +918,23 @@ PyTypeObject cups_OptionType =
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "PPD option",              /* tp_doc */
+    "PPD option\n"
+    "==========\n"
+    "  A PPD option.\n\n"
+    "@type conflicted: boolean\n"
+    "@ivar conflicted: whether this option is in conflict\n"
+    "@type keyword: string\n"
+    "@ivar keyword: the option keyword e.g. Duplex\n"
+    "@type defchoice: string\n"
+    "@ivar defchoice: the default option choice\n"
+    "@type text: string\n"
+    "@ivar text: the user-presentable option name e.g. Double-sided printing\n"
+    "@type ui: integer\n"
+    "@ivar ui: the option type; one of L{PPD_UI_BOOLEAN}, \n"
+    "L{PPD_UI_PICKONE}, L{PPD_UI_PICKMANY}\n"
+    "@type choices: list\n"
+    "@ivar choices: list of the option's choices\n"
+    "",                        /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
@@ -1031,7 +1111,18 @@ PyTypeObject cups_GroupType =
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "PPD option group",        /* tp_doc */
+    "PPD option group\n"
+    "================\n\n"
+    "  A PPD option group.\n\n"
+    "@type text: string\n"
+    "@ivar text: user-presentable group name\n"
+    "@type name: string\n"
+    "@ivar name: unique group name\n"
+    "@type options: L{Option} list\n"
+    "@ivar options: list of options in the group\n"
+    "@type subgroups: L{Group} list\n"
+    "@ivar subgroups: list of subgroups in the group\n"
+    "",                        /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
@@ -1169,7 +1260,18 @@ PyTypeObject cups_ConstraintType =
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "PPD constraint",          /* tp_doc */
+    "PPD constraint\n"
+    "==============\n\n"
+    "  A PPD constraint.\n\n"
+    "@type option1: string\n"
+    "@ivar option1: first option keyword\n"
+    "@type choice1: string\n"
+    "@ivar choice1: first option choice\n"
+    "@type option2: string\n"
+    "@ivar option2: second option keyword\n"
+    "@type choice2: string\n"
+    "@ivar choice2: secondoption choice\n"
+    "",                        /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
@@ -1307,7 +1409,18 @@ PyTypeObject cups_AttributeType =
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "PPD attribute",          /* tp_doc */
+    "PPD attribute\n"
+    "=============\n\n"
+    "  A PPD attribute.\n\n"
+    "@type name: string\n"
+    "@ivar name: attribute name\n"
+    "@type spec: string\n"
+    "@ivar spec: specifier string (if any)\n"
+    "@type text: string\n"
+    "@ivar text: human-readable text (if any)\n"
+    "@type value: string\n"
+    "@ivar value: attribute value\n"
+    "",                        /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
